/*
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation

  STARTER PROGRAM

 * Section AH
 *
 */


#include "gp142.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <time.h>
	/* 
	* <time.h> is included so the current time can be 
accessed
	* to initialize the random number generator.  If you 
don't
	* use random numbers (and you don't have to) you can 
either
	* ignore this #include or delete it.
	*/

//Structure type
typedef struct
{
	int x, y, direction, color, xspeed, yvelocity, height, 
on;
} vehicle;

typedef struct
{
	int a, b;
} duo;

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0


/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	7


/* Possible locations of mouse clicks: */

//Mouse click on car
#define CAR_CLICK 2

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0

#define FIELD_DURATION 12
#define ADDING 1+FIELD_DURATION //ticks for adding sequence important to be less than -1.


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER1 ((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_CENTER2 ((-GP142_YMAX + HORIZON) / 2)
#define HIGHWAY_CENTER3 (2*(-GP142_YMAX + HORIZON) /3)
#define HIGHWAY_WIDTH  10


/* Maximum number of vehicles that can appear in the 
animation */
#define CARS_PER_LANE	10
#define LANES 3
#define SPEED_LIMIT 20

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

//Traffic advisory board
#define BOARDX -GP142_XMAX/3
#define BOARDY (HORIZON + GP142_YMAX/5)  //Bottom of board
#define EDGE GP142_XMAX/44

#define CRASH -10


/***********************************************************
******************

     FUNCTION PROTOTYPES

************************************************************
*****************/

/* Initialize random number generator.  [Ignore or	
	*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

void draw_advisory(int left, int bottom, char *message, int length, int pos);
//Draw traffic advisory board

void update_message(int *pos,char *message, int length);

/* Draw the background, including the sun at coordinates 
sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y);

/* Draw the buttons */
void draw_buttons(int *adding);

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y, vehicle car[CARS_PER_LANE][LANES],
						  duo *active);

/* Process button click at location x, y.  Update car speed,  
*/
/* direction, color, and whether traffic is moving as 
needed. */
void handle_button(int x, int y, int *traffic_moving, vehicle car[CARS_PER_LANE][LANES], 
				   duo *active, int *adding);

void crashing_car(int x, int y, int color);

void space_hole(int x1, int y1, int x2, int y2, int color1, int color2);

void add_car(int x, int y, vehicle car[CARS_PER_LANE][LANES], int *adding, duo *active);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update all vehicle x, y coordinates, given current x, y 
*/
/* location, direction (+1 or -1), and speed in pixels per 
tick. */
void move_traffic(vehicle car[CARS_PER_LANE][LANES]);

/* Draw all vehicles given their current x, y coordinates 
and color */
void draw_traffic(vehicle car[CARS_PER_LANE][LANES]);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color);

/* Display error message if internal error is detected. */
void something_is_wrong (void);

void transporter(int x1, int y1, int x2, int y2, int *going);

void cool_sun(int x1, int y1, int x2, int y2);

void propulsion(int x1, int y1, int x2, int y2, int color1, int color2);


/***********************************************************
******************

     MAIN

************************************************************
*****************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int i, j, cars=1;         //Counters
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about vehicles in the animation:		
					*/
	/* [Only one car now - will need to be changed to add more]				*/
		
		
	vehicle car[CARS_PER_LANE][LANES];

	int traffic_moving;
	char message[256];
	int mess_pos=0;
	duo active= {-1, -1};
	int adding = FALSE;


	car[0][0].color = RED;
	car[0][0].direction = 1;
	car[0][0].height = 0;
	car[0][0].on = TRUE;
	car[0][0].x = 0;
	car[0][0].y = HIGHWAY_CENTER1;
	car[0][0].xspeed = 5;
	car[0][0].yvelocity = 0;

	for(i=0;i<CARS_PER_LANE;i++){
		for(j=0;j<LANES;j++)
		{
			car[i][j].on=0;
		}
	}

	initialize_random_numbers();

	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.		
	*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;



	traffic_moving = TRUE;
	strcpy(message, "Click on a button to control the cars.");
	
	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y);
	draw_advisory(BOARDX, BOARDY, message, strlen(message), mess_pos);
	draw_buttons(&adding);
	draw_traffic(car);
	

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should		*/
			/* react to keyboard events, insert something appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.		*/
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, 
				mouse_y, car, &active);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, 
							&traffic_moving, car, &active, &adding);
			} 
			else if (adding==TRUE && car[abs(active.a)%CARS_PER_LANE][abs(active.b)%LANES].on<=1)
			{
				add_car(mouse_x, mouse_y, car, &adding, &active);
			}
			//MORE IF MORE IF MORE IF MORE IF MORE IF
			
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun(&sun_x, &sun_y);
			update_message(&mess_pos, message, strlen(message));
			if (traffic_moving)
				move_traffic(car);
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */

		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_advisory(BOARDX, BOARDY, message, strlen(message), mess_pos);
		draw_buttons(&adding);
		draw_traffic(car);
		if(car[active.a][active.b].on > 1)
		transporter(car[active.a][active.b].x - CAR_WIDTH,
		car[active.a][active.b].y + 3*CAR_HEIGHT/2,
			car[active.a][active.b].x + CAR_WIDTH, car[active.a][active.b].y-CAR_HEIGHT/2, 
			&car[active.a][active.b].on);
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/***********************************************************
******************

     BACKGROUND LANDSCAPE

************************************************************
*****************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */

	/* Draw the sky */
	GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	cool_sun(sun_x-20, sun_y-20, sun_x+20, sun_y+20);
	
	/* Draw the ground, highway, and text legend */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX, GP142_XMAX, HORIZON, FILL);
	GP142_rectangleXY(LT_GRAY, -GP142_XMAX, HIGHWAY_CENTER1 - HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER1 + HIGHWAY_WIDTH, FILL);
	GP142_rectangleXY(LT_GRAY, 	-GP142_XMAX, HIGHWAY_CENTER2 - HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER2 + HIGHWAY_WIDTH,FILL);
	GP142_rectangleXY(LT_GRAY, -GP142_XMAX, HIGHWAY_CENTER3 - HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER3 + HIGHWAY_WIDTH, FILL);



	/* Draw the highway line */
	GP142_lineXY(YELLOW, -GP142_XMAX, HIGHWAY_CENTER1, GP142_XMAX,
			HIGHWAY_CENTER1, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX, HIGHWAY_CENTER2, GP142_XMAX,
			HIGHWAY_CENTER2, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX, HIGHWAY_CENTER3, GP142_XMAX,
			HIGHWAY_CENTER3, 1);

	
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
} /*end draw_landscape */



/***********************************************************
******************

     BUTTONS AND MOUSE CLICKS

************************************************************
*****************/


/* Draws the buttons */
void draw_buttons(int *adding )
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button 
text in button */
	int font_size = 12;				/* size of button 
text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "New Color");

	i -= BUTTON_HEIGHT;
	if(*adding!=0)
		GP142_printfXY(WHITE, text_offset, i, font_size, "Add Car");
	else
		GP142_printfXY(BLACK, text_offset, i, font_size, "Add Car");
	
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind 
*/
int  classify_mouse_click(int mouse_x, int mouse_y, vehicle car[CARS_PER_LANE][LANES],
						  duo *active)
{
	int i, j, car_click=0;
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else if(car[abs(active->a % CARS_PER_LANE)][abs(active->b % LANES)].on<=1)
	{
		for(i=0;i<CARS_PER_LANE;i++)
			for(j=0;j<LANES;j++)
			{
				if(mouse_x > car[i][j].x - CAR_WIDTH/2 &&
					mouse_x < car[i][j].x + CAR_WIDTH/2 &&
					mouse_y > car[i][j].y &&
					mouse_y < car[i][j].y + CAR_HEIGHT)
				{
					active->a = i;
					active->b = j;
					return CAR_CLICK;
				}
			}
	}

	return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,
	*/
/* direction, color, and whether traffic is moving as 
needed.	*/
void handle_button(int x, int y, int *traffic_moving, 
vehicle car[CARS_PER_LANE][LANES], duo *active, int *adding)
{
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button 
*/
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* go */
		*traffic_moving = TRUE;
		break;
	case 1:
		/* stop */
		*traffic_moving = FALSE;
		break;
		
	case 2:
		/* speed up */
		if(car[active->a][active->b].xspeed < SPEED_LIMIT)
			car[active->a][active->b].xspeed += 1;
		break;
	case 3:
		/* slow down */
		car[active->a][active->b].xspeed -= 1;
		if (car[active->a][active->b].xspeed < 0)
			car[active->a][active->b].xspeed  = 0;
		break;
		
	case 4:
		/* reverse */
		car[active->a][active->b].direction = -car[active->a][active->b].direction;
		break;
	case 5:
		/* change car color */
		car[active->a][active->b].color = (car[active->a][active->b].color +1) % MAX_COLORS;
		break;
	case 6:
		*adding = (*adding +1)%2; //turns adding off if clicked on again
		break;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/***********************************************************
******************

     TRAFFIC AND OTHER ANIMATED OBJECTS

************************************************************
*****************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, 
move it */
	/* to the other side and down a bit. */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}


/* Update all vehicle x, y coordinates, given current x, y 
*/
/* location, direction (+1 or -1), and speed in pixels per 
tick, */
void move_traffic(vehicle car[CARS_PER_LANE][LANES]) {
	/* Note: If there is more than one vehicle, this should 
*/
	/* be updated to adjust all vehicle coordinates */

	int trial_x, i, j, i2, distance;		/* new value 
for x coordinate */

	/* move vehicle.  If it moves off the edge, don't 
reverse direction */
	for(i=0;i<CARS_PER_LANE;i++)
	{
		for(j=0;j<LANES;j++)
		{
			if(car[i][j].on!=0)
			{
				trial_x = car[i][j].x + (car[i][j].direction * car[i][j].xspeed);
				if ((abs(trial_x) > GP142_XMAX ) && 
					trial_x*(car[i][j].direction)>0)
				{
					trial_x = -trial_x;
					car[i][j].x = trial_x;
				}
				else
					car[i][j].x = trial_x;
			}
				for(i2=i+1;i2<CARS_PER_LANE;i2++)
				{
					if(car[i2][j].on>0)
					{
						if(car[i][j].on!=0)
						{
							distance=car[i2][j].x-car[i][j].x;
							if(abs(distance)<2.5*CAR_WIDTH || 2*GP142_XMAX-abs(distance)<5*CAR_WIDTH/2)
							{
								if(2*GP142_XMAX-distance<5*CAR_WIDTH/2)
									distance=-(2*GP142_XMAX-distance);
								else if(2*GP142_XMAX-abs(distance)<5*CAR_WIDTH/2)
									distance=(2*GP142_XMAX-abs(distance));
								if((car[i2][j].direction* (car[i2][j].xspeed)-car[i][j].direction* car[i][j].xspeed)* (distance)<0)
								{
									if(abs(distance)-CAR_WIDTH!=0)
									 car[i2][j].yvelocity= 
									(abs(car[i2][j].direction*car[i2][j].xspeed- 
									car[i][j].direction*car[i][j].xspeed))* 
									(CAR_HEIGHT-car[i2][j].y)/(abs(distance-CAR_WIDTH)) ;
								}
								else
								{
									if(CAR_HEIGHT - car[i2][j].y!=0)
									car[i2][j].yvelocity= -
									((abs(car[i2][j].direction*car[i2][j].xspeed-car[i][j].direction*car[i][j].xspeed))*
									(abs(distance))/ abs(CAR_HEIGHT-car[i2][j].y));
								}
							}
						}
						if(car[i2][j].height >2*CAR_HEIGHT&&car[i2][j].yvelocity >0)
							car[i2][j].yvelocity =1;
						car[i2][j].y -=car[i2][j].height;
						car[i2][j].height+= car[i2][j].yvelocity;
						if(car[i2][j].height<0)
						{
							car[i2][j].height=0;
							car[i2][j].yvelocity=0;
						}
						car[i2][j].y += car[i2][j].height;
					
						if(abs(car[i2][j].height- car[i][j].height)<CAR_HEIGHT &&
							abs(distance)<CAR_WIDTH)
						{
							car[i2][j].on=CRASH;
							car[i][j].on=CRASH;
						}
					}
				}
		}
	}
}


/* Draw all vehicles given their current x, y coordinates 
and color */
void draw_traffic(vehicle car[CARS_PER_LANE][LANES]) 
{
	/* Note: needs to be changed if there is more than */
	/* one vehicle */
	int i, j;
	for(i=0;i<CARS_PER_LANE;i++)
		for(j=0;j<LANES;j++)
		{
			if(car[i][j].on>0)
			{
				/*if(car[i][j].on>1)
					propulsion(car[i][j].x-CAR_WIDTH/2, 
					car[i][j].y+car[i][j].height-CAR_HEIGHT/2, car[i][j].x+CAR_WIDTH/2, 
					car[i][j].y+car[i][j].height+CAR_HEIGHT/2, NAVY_BLUE, MAGENTA);*/
			if(car[i][j].yvelocity!=0||car[i][j].height>0)
					propulsion(car[i][j].x-CAR_WIDTH/2-10, 
					car[i][j].y-CAR_HEIGHT/5, car[i][j].x+CAR_WIDTH/2+10, 
					car[i][j].y+4*CAR_HEIGHT/5-7, NAVY_BLUE, MAGENTA);
				draw_car(car[i][j].x, car[i][j].y, 
				car[i][j].color);
			}
			if(car[i][j].on<0)
			{
				crashing_car(car[i][j].x, car[i][j].y, car[i][j].color);
				car[i][j].on++;
			}
		}
}



/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color) {
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
}



/***********************************************************
******************

     OTHER

************************************************************
*****************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}

void draw_advisory(int left, int bottom, char *message, int 
length, int pos)
{
	int right, top;
	right=left+2*GP142_XMAX/3;
	top=bottom+GP142_YMAX/6;
	GP142_rectangleXY(BLACK, left, bottom, right, top, FILL);


	GP142_printfXY(YELLOW, right-pos, bottom+2*EDGE,
					12, "%s", message);

	GP142_rectangleXY(WHITE, left, top, left + EDGE,
						HORIZON-GP142_YMAX/25, FILL);
	GP142_rectangleXY(WHITE, right, top, right - EDGE,
						HORIZON-GP142_YMAX/25, FILL);
	GP142_rectangleXY(WHITE, left+EDGE, top, left + 2*EDGE,
						HORIZON-GP142_YMAX/25, FILL);
	GP142_rectangleXY(WHITE, right-EDGE, top, right - 2*EDGE,
						HORIZON-GP142_YMAX/25, FILL);
	GP142_rectangleXY(FOREST_GREEN, right+1, top,
						GP142_XMAX, bottom, FILL);
	GP142_rectangleXY(OLIVE, left-1, top,
						-GP142_XMAX, bottom, FILL);
}

void update_message(int *pos,char *message, int length)
{
	if ((*pos) < 2*GP142_XMAX/3+length*5)
		(*pos)+=4;
	else
		*pos = 0;
}

void transporter(int x1, int y1, int x2, int y2, int *going)
{
	static int i;
	int j, k, temp, factor;
	long int random;
	if(x2-x1<0)
	{
		temp=x2;
		x2=x1;
		x1=temp;
	}
	if(y2-y1<0)
	{
		temp=y2;
		y2=y1;
		y1=temp;
	}
	if(i<FIELD_DURATION)
	{
		for(j=0;j<abs(y2-y1);j++)
		{
			for(k=(int)( (double)(x2-x1)/2.0*(double)sqrt(1.0-(double)4.0*((double)(y2-y1)/2.0 -(double)j)*
				((double)(y2-y1)/2.0-(double)j)/(double)(y2-y1)/(double)(y2-y1)));
			k > (int)(-(double)(x2-x1)/2.0*sqrt(1.0-(double)4.0*((double)(y2-y1)/2.0-(double)j)*
				((double)(y2-y1)/2-(double)j)/(double)(y2-y1)/(double)(y2-y1))) ;k--)
			{
				factor=(int) sqrt((j-(y2-y1)/2)*(j-(y2-y1)/2)
							+ k*k)+3;
				random=rand();
				if(random%(factor/2)==0)
					GP142_pixelXY(NAVY_BLUE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==1)
					GP142_pixelXY(BLUE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==2)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==3)
					GP142_pixelXY(YELLOW, (x2+x1)/2-k, y1+j);
				else 
				if(random%2==0&&sqrt(j*j+k*k)<5||random%4==1)
					GP142_pixelXY(ICE_BLUE, (x2+x1)/2-k, y1+j);


			}
		}
		i++;
	}
	else{
		*going = abs(*going)-i;  
		i=0;
	}
}
void cool_sun(int x1, int y1, int x2, int y2)
{
   int j, k, temp, factor;
	long int random;
	if(x2-x1<0)
	{
		temp=x2;
		x2=x1;
		x1=temp;
	}
	if(y2-y1<0)
	{
		temp=y2;
		y2=y1;
		y1=temp;
	}
		for(j=0;j<abs(y2-y1);j++)
		{
			for(k=(int)( (double)(x2-x1)/2.0*(double)sqrt(1-(double)4*
				((y2-y1)/2 -j)*((y2-y1)/2-j)/(double)(y2-y1)/(double)(y2-y1)));
			k>-(x2-x1)/2.0*sqrt(1-(double)4*((y2-y1)/2-j)*
				((y2-y1)/2-j)/(double)(y2-y1)/(double)(y2-y1))
			;k--)
			{
				factor=(int) sqrt((j-(y2-y1)/2)*(j-(y2-y1)/2)
							+ (k-(x2-x1)/2)*(k-(x2-x1)))+2;
				random=rand();
            if (random%2==0||random%3==1)
					GP142_pixelXY(YELLOW, (x2+x1)/2-k, y1+j);
				/*if(random%(factor/2)==0)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==1)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==2)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);*/
				else
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);
			}
		}
}

void add_car(int x, int y, vehicle 
car[CARS_PER_LANE][LANES], int *adding, duo *active)
{
	int i, j;
	if(abs(y-HIGHWAY_CENTER1)<HIGHWAY_WIDTH)
		j=0;
	else if(abs(y-HIGHWAY_CENTER2)<HIGHWAY_WIDTH)
		j=1;
	else if(abs(y-HIGHWAY_CENTER3)<HIGHWAY_WIDTH)
		j=2;
	else
		return;

	
	for(i=0;i<CARS_PER_LANE;i++)
	{
		if(car[i][j].on == 0)
		{
			*adding = 0;
			car[i][j].on =ADDING;
			active->a=i;
			active->b=j;
			car[i][j].x = x;
			car[i][j].y = HIGHWAY_CENTER1 - abs(j*(-GP142_YMAX + HORIZON)/6);
			car[i][j].color = rand()%24;
			car[i][j].direction = 1;
			car[i][j].height = 0;
			car[i][j].xspeed = 5;
			car[i][j].yvelocity = 0;
			return;
		}
	}
}

void crashing_car(int x, int y, int color)
{
	space_hole(x+CAR_WIDTH, y+CAR_WIDTH, x-CAR_WIDTH, y-CAR_WIDTH, color, ORANGE);
}

void propulsion(int x1, int y1, int x2, int y2, int color1, int color2)
{
	static int i;
	int j, color[2];
	color[0]=color1, color[1]=color2;
	if(i<10)
		i++;
	else
		i=2;
	for(j=1;j<=i+1;j++)
	{
		space_hole(x1-j*(x1-x2)/(2*(i+1)), y2+j*(y1-y2)/(2*(i+1)), x2+j*(x1-x2)/(2*(i+1)),
			y1-j*(y1-y2)/(2*(i+1)), color[j%2], ICE_BLUE);
	}
}

//void transport(void


void space_hole(int x1, int y1, int x2, int y2, int color1, int color2)
{
   int j, k, temp, factor;
	long int random;
	if(x2-x1<0)
	{
		temp=x2;
		x2=x1;
		x1=temp;
	}
	if(y2-y1<0)
	{
		temp=y2;
		y2=y1;
		y1=temp;
	}
		for(j=0;j<abs(y2-y1);j++)  /*Make an ellipse with sparkling pixels*/
		{
			for(k=(int)( (double)(x2-x1)/2.0*(double)sqrt(1-(double)4*((y2-y1)/2 -j)*
				((y2-y1)/2-j)/(double)(y2-y1)/(double)(y2-y1)));
			k>-(x2-x1)/2.0*sqrt(1-(double)4*((y2-y1)/2-j)*((y2-y1)/2-j)/(double)(y2-y1)/(double)(y2-y1))
			;k--)
			{
				factor= (int) sqrt((j-(y2-y1)/2)*(j-(y2-y1)/2)
							+ (k-(x2-x1)/2)*(k-(x2-x1)))+2;
				random=rand();
            if (random%2==0||random%3==1)
					GP142_pixelXY(color1, (x2+x1)/2-k, y1+j);
				/*if(random%(factor/2)==0)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==1)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);
				else if(random%(factor/2)==2)
					GP142_pixelXY(WHITE, (x2+x1)/2-k, y1+j);*/
				else
					GP142_pixelXY(color2, (x2+x1)/2-k, y1+j);
			}
		}
}

